<?php
if ( ! class_exists( 'Spai_Widget' ) ) {

    class Spai_Widget extends WP_Widget
    {
        CONST RELATED_POSTS_DEFAULT_HEADING = 'Related posts:';
        CONST RELATED_POSTS_DEFAULT_IMAGE = __SPAI_PUBLIC__ . 'img/default-picture.png';
        private $post_id;
        private $spai_options;
        private $token;
        private $count_posts = 0;
        private $max_ads_count = 0;
        private $asWidget;
        private $hash = '';

        public $content = null;

        public function __construct($asWidget = 1) {
            $this->asWidget = $asWidget;
            parent::__construct(
                'spai_widget',
                __( 'Spai - related posts', 'spai' ),
                array(
                    'customize_selective_refresh' => false,
                )
            );

            $this->post_id = get_the_ID();
            $this->spai_options = get_option( 'spai' );
            $this->token = $this->spai_options['api_key'];

            $this->setCountPosts();
            $this->setMaxAdsCount();
            $this->setHash();
        }

        // Сохранение настроек виджета (очистка)
        public function update( $new_instance, $old_instance) {
            $instance = array();

            $instance['display_type'] = ( !empty( $new_instance['display_type'] ) ) ? (int)$new_instance['display_type'] : 3;
            if( $instance['display_type'] > 5 || $instance['display_type'] < 1 ) { $instance['display_type'] = 3; }

            $instance['template'] = ( !empty( $new_instance['template'] ) ) ? (int)$new_instance['template'] : 2;
            if( $instance['template'] > 3 || $instance['template'] < 1 ) { $instance['display_type'] = 2; }

            return $instance;
        }

        // Вывод виджета
        public function widget( $args = [], $instance = []) {
            if ($this->asWidget === 1) {
                global $post;
                $this->post_id = isset($post) ? $post->ID : null;
            }
            $related_posts_as_html = '';
            $postsData = $this->getRelatedPosts();
            $relatedPosts = !empty($postsData['posts']) ? $postsData['posts'] : [];
            $ads = !empty($postsData['ads']) ? $postsData['ads'] : [];
            if (count($ads) > 0) {
                $ads = array_slice($ads, 0, $this->max_ads_count); //ограничиваем массив макс допустимым количеством рекламы
            }

            if( $postsData !== false and !empty( $relatedPosts ) ) {
                $send_impression = $this->sendImpression( $relatedPosts );

                $imp_id = null;
                if( !empty( $send_impression ) and $send_impression->result === true) {
                    $imp_id = (int) $send_impression->data->imp_id;
                }

                $data = $this->prepareDataToWidget( $relatedPosts, $ads );

                $related_posts_as_html = $this->widgetAsHtml( $data, $imp_id, $instance );
            }

            if ($this->asWidget === 1) {
                echo $related_posts_as_html;
            } else {
                $this->content = $related_posts_as_html;
                return $this->content;
            }
        }

        // html форма настроек виджета в Админ-панели
        function form( $instance ) {
            $display_type = ! empty( $instance['display_type'] ) ? $instance['display_type'] : 3;
            $template = ! empty( $instance['template'] ) ? $instance['template'] : 2;
            ?>
                <div class="mt20 template-division">
                    <label class="spai-input_label" for="<?php echo esc_attr( $this->get_field_id( 'template' ) ); ?>"><?php _e( 'Template', SPAI_PLUGIN_NAME );?></label>
                    <div class="spai-input_division">
                        <select
                                name="<?php echo esc_attr( $this->get_field_name( 'template' ) ); ?>"
                                id="<?php echo esc_attr( $this->get_field_id( 'template' ) ); ?>">
                            <option value="1" <?php if( $template === 1 ) {echo 'selected';} ?>><?php esc_attr_e( 'Template 1', SPAI_PLUGIN_NAME );?></option>
                            <option value="2" <?php if( $template === 2 ) {echo 'selected';} ?>><?php esc_attr_e( 'Template 2', SPAI_PLUGIN_NAME );?></option>
                            <option value="3" <?php if( $template === 3 ) {echo 'selected';} ?>><?php esc_attr_e( 'Stories', SPAI_PLUGIN_NAME );?></option>
                        </select>
                    </div>
                </div>

                <div class="mt20 display_type-division">
                    <label class="spai-input_label" for="<?php echo esc_attr( $this->get_field_id( 'display_type' ) ); ?>">
                        <?php _e( 'Display type', SPAI_PLUGIN_NAME );?>
                    </label>
                    <div class="spai-input_division">
                        <select
                                name="<?php echo esc_attr( $this->get_field_name( 'display_type' ) ); ?>"
                                id="<?php echo esc_attr( $this->get_field_id( 'display_type' ) ); ?>">
                            <option value="1" <?php if( $display_type === 1 ) {echo 'selected';} ?>><?php esc_attr_e( '6 posts in 1 row', SPAI_PLUGIN_NAME );?></option>
                            <option value="2" <?php if( $display_type === 2 ) {echo 'selected';} ?>><?php esc_attr_e( '4 posts in 1 row', SPAI_PLUGIN_NAME );?></option>
                            <option value="3" <?php if( $display_type === 3 ) {echo 'selected';} ?>><?php esc_attr_e( '3 posts in 1 row', SPAI_PLUGIN_NAME );?></option>
                            <option value="4" <?php if( $display_type === 4 ) {echo 'selected';} ?>><?php esc_attr_e( '4 posts in 2 rows', SPAI_PLUGIN_NAME );?></option>
                            <option value="5" <?php if( $display_type === 5 ) {echo 'selected';} ?>><?php esc_attr_e( '3 posts in 2 rows', SPAI_PLUGIN_NAME );?></option>
                        </select>
                    </div>
                </div>
            <?php
        }

        private function getRelatedPosts()
        {
            if (empty($this->token)) {
                return false;
            }
            $api = new Spai_Api( $this->token );

            $answer = $api->getRelatedPosts( $this->post_id, $this->count_posts );
            //echo '<pre>';var_dump($answer);

            if( is_wp_error( $answer ) ) {
                return false;
            }
            if (!isset($answer['response']['code']) or $answer['response']['code'] !== 200) {
                return false;
            }

            $json =  json_decode( json_decode( $answer['body'] ) );
            if(!empty($json->result) and $json->result === true ) {
                $relatedPosts = $json->data->posts;
                $ads = $json->data->ads;
                return [
                    'posts' => $relatedPosts,
                    'ads' => $ads
                ];
            }

            return false;
        }

        private function widgetAsHtml( $data, $imp_id, $instance )
        {
            if ($this->asWidget === 1) {
                $template = ( isset($instance['template']) ) ? (int)$instance['template'] : 2;
            } else {
                $template = ( $this->spai_options['template'] ) ? (int)$this->spai_options['template'] : 2;
            }
            require_once plugin_dir_path( __FILE__ ) . 'widget-templates/template' . $template .'.php';
            $templateName = 'template' . $template;
            /** @var templateAbstract $template */
            $templateClass = new $templateName(
                $this->spai_options,
                $data,
                $imp_id,
                $this->post_id,
                $this->hash
            );
            return $templateClass->run();
        }

        private function sendImpression( $related_posts )
        {

            if ( !empty( $_SERVER['HTTP_CLIENT_IP'] ) ) {
                $ip = $_SERVER['HTTP_CLIENT_IP'];
            } elseif ( !empty( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) {
                $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
            } else {
                $ip = $_SERVER['REMOTE_ADDR'];
            }

            $user_agent = $_SERVER['HTTP_USER_AGENT'];

            $api = new Spai_Api( $this->token );
            $data = [
                'post_id' => $this->post_id,
                'recommended_posts' => $related_posts,
                'ip' => $ip,
                'user_agent' => $user_agent
            ];
            $answer = $api->sendImpressionData( $data );
            $json =  json_decode( json_decode( $answer['body'] ) );
            return $json;
        }

        private function prepareDataToWidget($relatedPosts, $ads)
        {
            $result = [];
            $adsCount = count($ads);
            $countAllElements = $adsCount + count($relatedPosts);

            if ($countAllElements > $this->count_posts) {
                $iMax = $this->count_posts;
            } else {
                $iMax = $countAllElements;
            }

            for ($i = 1; $i <= $iMax; $i++) {
                if ($i === 2 and $adsCount > 0 and !empty(array_values($ads)[0])) {
                    $adsCurrent = array_values($ads)[0];
                    $currentElement = [
                        'type' => 'ads',
                        'external_id' => $adsCurrent->id,
                        'url' => $adsCurrent->link,
                        'img' => $adsCurrent->image,
                        'title' => $adsCurrent->title
                    ];
                } elseif ($i === 4 and $adsCount > 0 and !empty(array_values($ads)[1])) {
                    $adsCurrent = array_values($ads)[1];
                    $currentElement = [
                        'type' => 'ads',
                        'external_id' => $adsCurrent->id,
                        'url' => $adsCurrent->link,
                        'img' => $adsCurrent->image,
                        'title' => $adsCurrent->title
                    ];
                } else {
                    $postCurrent = array_shift($relatedPosts);
                    $currentElement = [
                        'type' => 'post',
                        'url' => $postCurrent->post_url,
                        'img' => $postCurrent->img_url,
                        'title' => $postCurrent->title,
                        'external_id' => $postCurrent->external_id,
                        'categories' =>  $postCurrent->categories
                    ];
                }
                $result[] = (object)$currentElement;
            }
            //echo '<pre>';var_dump($result);echo '</pre>';
            return $result;
        }

        private function setCountPosts()
        {
            $template = ( $this->spai_options['template'] ) ? (int)$this->spai_options['template'] : 1;
            $display_type = (int)$this->spai_options['display_type'];
            if(empty($display_type)) {
                $display_type = 3;
            }
            if ($template === 3) {
                $this->count_posts = 12;
            } elseif (in_array($display_type, array(1, 5), true)) {
                $this->count_posts = 6;
            } elseif ($display_type === 2) {
                $this->count_posts = 4;
            } elseif ($display_type === 3) {
                $this->count_posts = 3;
            } elseif ($display_type === 4) {
                $this->count_posts = 8;
            } else {
                $this->count_posts = 6;
            }
        }

        private function setMaxAdsCount()
        {
            $display_type = (int)$this->spai_options['display_type'];
            if(empty($display_type)) {
                $display_type = 3;
            }

            if (in_array($display_type, array(1, 5), true)) {
                $this->max_ads_count = 2;
            } elseif ($display_type === 2) {
                $this->max_ads_count = 1;
            } elseif ($display_type === 3) {
                $this->max_ads_count = 1;
            } elseif ($display_type === 4) {
                $this->max_ads_count = 3;
            } else {
                $this->max_ads_count = 2;
            }
        }

        private function setAdsNotifierHtml()
        {
            $html = '<div class="spai-ads-notifier">';
                $html .= '<a class="spai-ads-notifier-block">Реклама</a>';
            $html .= '</div>';
            return $html;
        }

        private function setHash()
        {
            $length = 10;
            $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
            $charactersLength = strlen($characters);
            $randomString = '';
            for ($i = 0; $i < $length; $i++) {
                $randomString .= $characters[mt_rand(0, $charactersLength - 1)];
            }

            $this->hash = $randomString;
        }
    }

    function spai_register_widget()
    {
        register_widget( Spai_Widget::class );
    }
}
