<?php

class templateAbstract
{
    CONST RELATED_POSTS_DEFAULT_HEADING = 'Related posts:';
    CONST RELATED_POSTS_DEFAULT_IMAGE = __SPAI_PUBLIC__ . 'img/default-picture.png';

    protected $titleMaxDefault = 50;

    public function __construct(
        $my_plugin_options,
        $data,
        $imp_id,
        $post_id,
        $hash
    )
    {
        $this->my_plugin_options = $my_plugin_options;
        $this->data = $data;
        $this->imp_id = $imp_id;
        $this->post_id = $post_id;
        $this->hash = $hash;
    }

    public function run()
    {
        $template = ( $this->my_plugin_options['template'] ) ? (int)$this->my_plugin_options['template'] : 1;
        $show_category = ( $this->my_plugin_options['show_category'] ) ? (bool)$this->my_plugin_options['show_category'] : false;
        $display_type = (int)$this->my_plugin_options['display_type'];
        if (empty($display_type)) {$display_type = 3;}
        $html = '';
        $main_additional_classes = [];
        $main_additional_classes[] = 'spai-dt' . $display_type;
        $main_additional_classes[] = 'spai-tmpl' . $template;
        $main_additional_classes[] = ($show_category === true) ? 'with_cat' : '';

        if( $this->data ) {

            $html .= '<div class="spai-widget_content '.implode(' ', $main_additional_classes).'">';
            $html .= $this->setHeadingHtml();
            $html .= '<div class="spai-widget-posts" for_post_id="' . $this->post_id . '" imp_id="' . $this->imp_id . '">';
            foreach ($this->data as $item ) {
                if ($item->type === 'ads') {
                    $extIdHtml = '';
                } else {
                    $extIdHtml = ' related_post_id="' . $item->external_id . '"';
                }
                $html .= '<div class="spai-one_related_post" '.$extIdHtml.'>';
                    $html .= $this->setUrlHtml(
                        $item->url,
                        $this->setImgHtml( $item->img ),
                        $item->type
                    );

                    $title = $this->setTitleMax( $item->title );
                    $titleUrl= $this->setUrlHtml($item->url, $title, $item->type);
                    $html .= $this->setTitleHtml($titleUrl);

                    if ($item->type === 'post') {
                        if ($show_category === true) {
                            $html .= $this->setCategoriesHtml($item->categories);
                        }
                    } else {
                        $html .= $this->setAdsNotifierHtml();
                    }

                $html .= '</div>';

            }
            $html .= '</div>';
            $html .= '</div>';
        }

        return $html;
    }

    protected function setHeadingHtml()
    {
        $heading = ( $this->my_plugin_options['heading'] ) ? esc_attr( $this->my_plugin_options['heading'] ) : self::RELATED_POSTS_DEFAULT_HEADING;

        return '<h3 class="spai-widget-header">' . $heading . '</h3>';
    }

    protected function setAdsNotifierHtml()
    {
        $styles = [];
        if ( !empty( $this->my_plugin_options['category_color'] ) ) {
            $styles[] = 'color: ' . esc_attr( $this->my_plugin_options['category_color'] );
        }
        if ( !empty( $this->my_plugin_options['category_background_color'] ) ) {
            $styles[] = 'background-color: ' . esc_attr( $this->my_plugin_options['category_background_color'] );
            $styles[] = 'border: 1px solid ' . esc_attr( $this->my_plugin_options['category_background_color'] );
        }

        $html = '<div class="spai-ads-notifier">';
        $html .= '<span class="spai-ads-notifier-block" style="' . implode(';', $styles) . '">Реклама</span>';
        $html .= '</div>';
        return $html;
    }

    protected function setUrlHtml( $url, $urlHtml, $type )
    {
        $html = '';
        $rel = '';

        if ($type === 'ads') {
            $html .= '<noindex>';
            $rel = 'rel="sponsored nofollow"';
        }

        $html .= '<a href="' . esc_url( $url ) . '" '.$rel.'>';
        $html .= $urlHtml;
        $html .= '</a>';

        if ($type === 'ads') {
            $html .= '</noindex>';
        }
        return $html;
    }

    protected function setImgHtml( $img_url )
    {
        $img_url = $this->setImgUrl( $img_url );

        $img_additional_classes = [];
        $img_additional_classes[] = ((bool)$this->my_plugin_options['effect_of_increasing_the_image_size'] === true) ? 'increase' : '';

        return '<div class="spai-related_post_img ' . implode(' ', $img_additional_classes) . '" style="background-image: url(' . $img_url . ')"></div>';
    }

    protected function setImgUrl( $img_url )
    {
        return $img_url ?
            esc_url( $img_url ) :
            (
            $this->my_plugin_options['default_image']  ?
                esc_url( $this->my_plugin_options['default_image'] ) :
                self::RELATED_POSTS_DEFAULT_IMAGE
            );
    }

    protected function setTitleHtml( $title )
    {
        $styles = [];
        if ($this->my_plugin_options['heading_color']) {
            $styles[] = 'color: ' . esc_attr( $this->my_plugin_options['heading_color'] );
        }

        return '<div class="spai-related_post_title" style="' . implode(';', $styles) . '">' . $title . '</div>';
    }

    protected function setTitleMax( $title )
    {
        $title_max = ( $this->my_plugin_options['title_max'] ) ? (int)$this->my_plugin_options['title_max'] : $this->titleMaxDefault;

        $title_to_show = ( mb_strlen( $title, 'UTF-8' ) > $title_max ) ?
            mb_strimwidth( $title, 0, $title_max, "...", 'UTF-8' ) :
            $title;
        return esc_html( $title_to_show );
    }

    protected function setCategoriesHtml($categories) {
        $html = '';

        if(count($categories) > 0) {
            $styles = $this->getCategoryStyles();

            $html .= '<div class="spai-categories">';
            $category_links_html = [];
            foreach ($categories as $category) {
                $category_links_html[] = $this->setOneCategoryHtml($category, $styles);
            }
            $html .= implode(' ', $category_links_html);
            $html .= '</div>';
        }
        return $html;
    }

    protected function setOneCategoryHtml($category, $styles) {
        $category_link = get_category_link( $category->external_id );
        return '<a class="spai-one_category" style="' . implode(';', $styles) . '" href="' . esc_url( $category_link ) . '">'
                    . $category->name
                . '</a>';
    }

    protected function getCategoryStyles()
    {
        $styles = [];
        if ($this->my_plugin_options['category_color']) {
            $styles[] = 'color: ' . esc_attr( $this->my_plugin_options['category_color'] );
        }
        if ($this->my_plugin_options['category_background_color']) {
            $styles[] = 'background-color: ' . esc_attr( $this->my_plugin_options['category_background_color'] );
            $styles[] = 'border: 1px solid ' . esc_attr( $this->my_plugin_options['category_background_color'] );
        }
        return $styles;
    }
}
