<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://my-plugin/author
 * @since      1.0.0
 *
 * @package    Spai
 * @subpackage Spai/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @property  Spai_Public
 * @package    Spai
 * @subpackage Spai/public
 * @author     Admin <admin@spai.site>
 */
class Spai_Public {

    public $api;

    /**
     * @var false|mixed|null
     */
    private $my_plugin_options;

    /**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		$this->my_plugin_options = get_option( SPAI_PLUGIN_NAME );

        $this->api = new Spai_Api( $this->my_plugin_options['api_key'] );

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function spai_enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Spai_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Spai_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
        wp_enqueue_style( SPAI_PLUGIN_NAME . '-short_codes', plugin_dir_url( __FILE__ ) . 'css/spai-short_codes.css', array(), SPAI_VERSION, 'all' );
		wp_enqueue_style( SPAI_PLUGIN_NAME . '-public', plugin_dir_url( __FILE__ ) . 'css/spai-public.css', array(), SPAI_VERSION, 'all' );

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function spai_enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Spai_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Spai_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_script( SPAI_PLUGIN_NAME . '-public', plugin_dir_url( __FILE__ ) . 'js/spai-public.js', array( 'jquery' ), SPAI_VERSION, false );
		wp_enqueue_script( SPAI_PLUGIN_NAME . '-jquery-mobile', plugin_dir_url( __FILE__ ) . 'libs/jquery-mobile/1.4.5/jquery.mobile.custom.min.js', array( 'jquery' ), SPAI_VERSION, false );

	}

	/**
	 * The function of adding text to the footer
	 *
	 * @since    1.0.0
	 */

	public function spai_add_widget_after_post( $content ){
        // Return if it's not in the loop or in the main query.
        if ( ! in_the_loop() && ! is_main_query() ) {
            return $content;
        }

        if ( doing_filter('get_the_excerpt') ) {
            return $content;
        }

        if( is_single() ) {
            $widget = new Spai_Widget(0);
            $content .= $widget->widget();
        }

        return $content;
	}

    /**
     * The function of adding widget to the content
     *
     * @since    1.3.4
     */
    public function spai_register_widget() {
        register_widget( 'Spai_Widget' );
    }

	public function checkSpaiPostsDataSync()
    {
        global $wpdb;
        $table_name = $wpdb->prefix . "spai_posts_data_sync";
        $total_posts_in_spai_posts_data_sync = $wpdb->get_results( "SELECT * FROM " . $table_name );
        $check_posts_sync = $wpdb->get_results( "SELECT * FROM " . $table_name . ' WHERE is_sync = false LIMIT 3' );

        $data = array(
            'posts' => array(),
            'total_posts' => count( $total_posts_in_spai_posts_data_sync )
        );

        if( $check_posts_sync ) {
            foreach ( $check_posts_sync as $check_post_sync ) {
                $the_post = get_post( $check_post_sync->post_id );
                if( isset( $the_post ) && $the_post->ID >0 && $the_post->post_status === 'publish' ) {
                    $tags = wp_get_post_tags( $the_post->ID);
                    $post_tags = array();
                    if( $tags ) {
                        foreach ( $tags as $tag ) {
                            $post_tags[] = array(
                                'id' => $tag->term_id,
                                'name' => $tag->name
                            );
                        }
                    }

                    $categories = wp_get_post_categories( $the_post->ID, array('fields' => 'all') );
                    $post_categories = array();
                    if( $categories ) {
                        foreach ( $categories as $category ) {
                            $post_categories[] = array(
                                'id' => $category->term_id,
                                'name' => $category->name
                            );
                        }
                    }

                    $data['posts'][] = array(
                        'post_id' => $the_post->ID,
                        'title' => $the_post->post_title,
                        'post_url' => get_permalink( $the_post->ID ),
                        'img_url' => get_the_post_thumbnail_url( $the_post->ID, 'medium' ),
                        'post_status' => $the_post->post_status,
                        'tags' => $post_tags,
                        'categories' => $post_categories
                    );
                } else {
                    $wpdb->delete( $table_name, [ 'post_id' => $check_post_sync->post_id ] );
                }
            }

            //echo '<pre>';var_dump($data);exit;
            $result = $this->api->sendPostData( $data );
            $result = json_decode( json_decode( $result ) );
            if( isset( $result->result ) && $result->result == true ) {
                $result_data = $result->data;
                foreach ( $result_data as $result_data_one ) {
                    if( $result_data_one->is_synced == true ) {
                        $wpdb->query( $wpdb->prepare( "UPDATE $table_name SET is_sync = %s WHERE post_id = %s", true, $result_data_one->post_id ) );
                    }
                }
            }
        }
    }

    public function spai_save_related_post_click()
    {
        $post_id = ( int ) $_POST['spai_post_id'];
        $clicked_related_post_id = ( int ) $_POST['spai_clicked_related_post_id'];
	    $imp_id = ( int ) $_POST['spai_imp_id'];

	    if (!empty($_SERVER['HTTP_CLIENT_IP'])) {
		    $ip = $_SERVER['HTTP_CLIENT_IP'];
	    } elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
		    $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
	    } else {
		    $ip = $_SERVER['REMOTE_ADDR'];
	    }

	    $user_agent = $_SERVER['HTTP_USER_AGENT'];

        $data = array(
            'post_id' => $post_id,
            'clicked_related_post_id' => $clicked_related_post_id,
            'imp_id' => $imp_id,
	        'ip' => $ip,
	        'user_agent' => $user_agent
        );
        $result = $this->api->sendClickToRelatedPost( $data );
        echo $result;
    }

    public function spai_define_javascript_ajaxurl() {
        if ( is_single() ) {
            ?>
            <script type="text/javascript">
                var spai_ajaxurl = '<?php echo admin_url('admin-ajax.php'); ?>';
            </script>
            <?php
        }
    }

}
